@@ -1,14 +1,12 @@
 use crate::sys::{MemoryType, Pages, TableType};
-use loupe::MemoryUsage;
 use std::ptr::NonNull;
 use std::sync::Arc;
 use target_lexicon::PointerWidth;
 use wasmer_compiler::Target;
-use wasmer_engine::Tunables;
 use wasmer_vm::MemoryError;
 use wasmer_vm::{
-    LinearMemory, LinearTable, Memory, MemoryStyle, Table, TableStyle, VMMemoryDefinition,
-    VMTableDefinition,
+    LinearMemory, LinearTable, Memory, MemoryStyle, Table, TableStyle, Tunables,
+    VMMemoryDefinition, VMTableDefinition,
 };
 
 /// Tunable parameters for WebAssembly compilation.
@@ -19,7 +17,7 @@ use wasmer_vm::{
 /// implementation or use composition to wrap your Tunables around
 /// this one. The later approach is demonstrated in the
 /// tunables-limit-memory example.
-#[derive(Clone, MemoryUsage)]
+#[derive(Clone)]
 pub struct BaseTunables {
     /// For static heaps, the size in wasm pages of the heap protected by bounds checking.
     pub static_memory_bound: Pages,
